#-*- coding: utf-8 -*-

# Copyright 2010 Calculate Ltd. http://www.calculate-linux.org
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from cl_builder import cl_builder, __app__, __version__
from cl_opt import opt
from cl_share_cmd import share_cmd
import os
from os import path
from os import access,R_OK
import re
import sys
from types import ListType

from cl_lang import lang
lang().setLanguage(sys.modules[__name__])

DESCRIPTION = _("The Calculate Linux image builder")
CMD_OPTIONS = [{'shortOption':"f",
                'longOption':"force",
                'help':_("no questions during the creating process")
               }]
USAGE = _("%prog [options] iso|squash")

class image_cmd(share_cmd):
    """Class for work with cl_builder by console"""
    def __init__(self):
        self.optobj = opt(package=__app__,
                          version=__version__,
                          description=DESCRIPTION,
                          usage=USAGE,
                          option_list= CMD_OPTIONS + opt.variable_control +
                          opt.color_control,
                          check_values=self.checkOpts)
        self.logicObj = cl_builder()
        self.commands = ["iso","squash"]
        #self.optionsInitrdIncompatible = ["o","no_clean","m","mdadm","lvm",
        #                                  "k", "e","dmraid","c","ebuild",
        #                                  "symlink"]

    def isScratch(self):
        """Detect scartch system"""
        if self.logicObj.clVars.Get('os_scratch') == 'on':
            return True
        else:
            self.printERROR(_("You should load system in the builder mode."))
            return False

    def checkOpts(self, values, args):
        """Check values all specified options."""
        if len(args) > 1:
            self.optobj.error(_("unrecognized option") + ": %s"% "".join(args))
        if len(args) < 1 and not values.v:
            self.optobj.error(_("missing command argument"))
        if args and not args[0] in self.commands:
            self.optobj.error(_("invalid command: '%s' (choose from %s)")%
                    (args[0],", ".join(map(lambda x:"'%s'"%x,self.commands))))
        # for work with v param
        if args:
            self.logicObj.clVars.Set('cl_action',args[0],True)

        if args and args[0] == "squash":
            self.logicObj.clVars.Set('cl_builder_iso_path','/mnt/flash',True)
        self.optobj.checkVarSyntax(values)
        return (values, args)

    def performActionByCommand(self,options,args):
        action = {'iso':self.makeIso,
                  'squash':self.makeSquash}
        action[args[0]](options)

    def makeIso(self,options):
        """Make iso"""
        if not self.logicObj.makeIsoImage(options.f):
            return False
        return True

    def makeSquash(self,options):
        """Make squash (rescratch)"""
        if not self.logicObj.makeRescratch(options.f):
            return False
        return True

    def createImagePaths(self):
        """Create path, which contains images"""
        imagepath = self.logicObj.clVars.Get('cl_builder_image_path')
        if type(imagepath) != ListType:
            imagepath = [imagepath]
        for pathname in imagepath:
            try:
                if not path.lexists(pathname):
                    os.makedirs(pathname,mode=0755)
            except:
                pass

    def checkChrootStatus(self):
        if self.logicObj.clVars.Get('cl_chroot_status') == 'off':
            return True
        else:
            self.printERROR(_("This program can't be run from Scratch layer"))
            return False
